<?php

namespace App\Http\Controllers;

use App\Http\Resources\UserResource;
use App\Jobs\SendOtpJob;
use App\Models\AppNotification;
use App\Models\RewardPoint;
use App\Models\Setting;
use App\Models\User;
use App\Services\Interface\UserInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Foundation\Auth\ThrottlesLogins;
use Laravel\Socialite\Facades\Socialite;


class AuthController extends Controller
{

    // public function __construct()
    // {
    //     $this->middleware('throttle:5,1')->only('login', 'verify.otp');
    // }

    // use ThrottlesLogins;

    public function getAuthUser()
    {
        try {
            if (!Auth::check()) return Base::error('Not Authenticated'); // DONE

            $data = new UserResource(Auth::user());

            return Base::success('Authenticated user fetched', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function login(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'email' => 'required|string|email',
                'password' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = User::where('email', strtolower($request->email))->first();
            if (!$user) return Base::error("User not found"); // DONE

            $login = $request->validate([
                'email' => 'required|string',
                'password' => 'required',
            ]);

            $res = $i_user->login($login, request()->header('app_role'));
            return $res->success ? Base::success($res->message, $res->data) : Base::error($res->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function register(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'name' => 'required|string|min:4',
                'email' => 'required|string|email|unique:users',
                'role' => 'required|string|in:customer,shop',
                'password' => 'required|confirmed|min:6',
            ]);
            if ($validator->fails()) return Base::validation($validator);


            $user_exist = User::where('email', strtolower($request->email))->first();
            if ($user_exist) return Base::error("User already exist"); // DONE

            $input = $request->all();
            $settings = Setting::all();

            if (isset($request->refer_code)) {
                $is_referral = $settings->where('type', 'is_referral')->first()->value;
                if ($is_referral == 0) return Base::error("Referral is disabled"); // DONE
                $refer_user = User::where('refer_code', $request->refer_code)->first();
                if (!$refer_user) return Base::error("Wrong refer code"); // DONE
            }

            $res = $i_user->createUser($input);
            if (!$res->success) return Base::error($res->message);

            if ($request->role === 'shop') {
                $res2 = $i_user->createShop($res->data->id);
                if (!$res2->success) {
                    $res->data->forceDelete();
                    return Base::error($res2->message);
                }
            }

            $user = $res->data;

            $send_otp = $i_user->sendOtp($user);
            if (!$send_otp->success) return Base::error($send_otp->message);

            $accessToken = $user->createToken('authToken')->accessToken;
            $data = [
                'token' => $accessToken,
                'user' => new UserResource($user),
            ];
            // $data = new UserResource(Auth::user());

            return Base::success("User Saved, Please verify your account", $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function resendOTP(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'email' => 'required|email',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = User::where('email', strtolower($request->email))->first();
            if (!$user) return Base::error("User not found");

            // if ($user->login_type != "normal") return Base::error("Reset password option is not available for this user"); // DONE


            // ! Checking app role
            if ($user->role !== request()->header('app_role')) return Base::error("User does not exist"); // DONE

            if ($user->otp && $user->otp_created_at) {
                $dif = Base::now()->diffInSeconds($user->otp_created_at);
                if ($dif < 120) return Base::error('Already sent OTP', (120 - $dif), 'otp_exist'); // DONE
            }

            $send_otp = $i_user->sendOtp($user);
            if (!$send_otp->success) return Base::error($send_otp->message);

            $data = new UserResource($user);
            if ($request->forget) $data['forget_password'] = true;

            return Base::success('OTP Code has been sent to your email', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function verifyOTP(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'otp' => 'required|numeric',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            if (Auth::check()) {
                $user = Auth::user();
            } else {
                $validator = Validator::make($request->all(), [
                    'email' => 'required|email',
                ]);
                if ($validator->fails()) return Base::validation($validator);
                $user = User::where('email', strtolower($request->email))->first();
            }
            if (!$user) return Base::error("User not found");

            if ($user->otp && $user->otp_created_at) {
                $dif = Base::now()->diffInSeconds($user->otp_created_at);
                if ($dif > 120) return Base::error('OTP Code time expired'); // DONE
            }

            if (!Hash::check($request->otp, $user->otp)) return Base::error('OTP does not match'); // DONE

            if ($request->forget) {
                $accessToken = rand(10000000, 99999999);
                $user->otp = $accessToken;
                $user->otp_created_at = Base::now();

                $data = [
                    'token' => $accessToken,
                    'forget_password' => true,
                    'user' => new UserResource($user),
                ];
            } else {

                if($user->refer_by != null){
                    $settings = Setting::all();
                    $is_referral = $settings->where('type', 'is_referral')->first()->value;
                    if ($is_referral == 1) {
                        $referral_value = $settings->where('type', 'referral_point')->first()->value;
                        $refer_user = User::where('refer_code', $user->refer_by)->first();
                        $refer_user->point = $refer_user->point + $referral_value;
                    

                        $reward_point = new RewardPoint();
                        $reward_point->user_id = $refer_user->id;
                        $reward_point->point = $referral_value;
                        $reward_point->type = 'referral';
                        $reward_point->description = 'Referral point';
                        $reward_point->save();

                        $refer_user->save();

                        $refer_notification = [
                            'from_id' => $user->id,
                            'to_id' => $refer_user->id,
                            'type' => 'referral',
                            'title_key' => 'referral_title',
                            'description_key' => 'referral_description',
                            'value' => $referral_value,
                        ];
                        NotificationController::setNotification($refer_notification);
                        
                    }
                }

                $accessToken = $user->createToken('authToken')->accessToken;
                $user->is_verified = true;
                $user->otp = null;
                $user->otp_created_at = null;

                $data = [
                    'token' => $accessToken,
                    'user' => new UserResource($user),
                ];
            }

            $user->save();
            return Base::success('OTP verified', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function resetPassword(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'email' => 'required|string|email',
                'token' => 'required',
                'password' => 'required|confirmed|min:6',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = User::where('email', strtolower($request->email))->first();
            if (!$user) return Base::error("User not found");

            if ($user->otp != $request->token) return Base::error('Reset token does not matched'); // DONE

            $user->password = Hash::make($request->password);
            $user->otp = null;
            $user->otp_created_at = null;
            $user->save();

            Auth::login($user);

            $token = Auth::user()->createToken('authToken')->accessToken;

            $data = [
                'token' => $token,
                'user' => new UserResource($user),
            ];

            return Base::success('Password has been changed', $data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function logout(Request $request)
    {
        try {
            if (!Auth::user()) return Base::error('User not logged in'); // DONE
            $user_app = AppNotification::where(['user_id' => Auth::user()->id, 'device_token' => $request->device_token])->first();
            if ($user_app != null) {
                $user_app->subscription = false;
                $user_app->save();
            }
            $user = Auth::user()->token();
            $user->revoke();
            return Base::success('User successfully logged out'); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    //* TEST FUNCTION

    public function socialLogin(UserInterface $i_user, Request $request)
    {
        $data = $i_user->socialLogin($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function referralUser(UserInterface $i_user, Request $request)
    {
        $data = $i_user->referralUser($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }

    public function vIDVerify(UserInterface $i_user, Request $request){
        $validator = Validator::make($request->all(), [
            'vid_front' => 'required',
            'vid_back' => 'required',
            'type' => 'required|in:mncard,rcard,driving,passport', //* mnid = my number card, rcard= residency card, driving license, passport
        ]);
        if ($validator->fails()) return Base::validation($validator);

        $data = $i_user->vIDVerify($request);
        return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
    }
}
