<?php

namespace App\Http\Controllers;

use App\Http\Requests\TogglePushOrEmailNotificationRequest;
use App\Models\Setting;
use App\Models\User;
use App\Models\UserInfo;
use App\Services\Interface\UserInterface;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Routing\Route;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class ProfileController extends Controller
{
    private $user_interface;

    public function __construct(UserInterface $user_interface)
    {
        $this->user_interface = $user_interface;
    }


    /**
     * Function index
     * Type: GET
     * Return: Auth user profile info.
     * Note: 
     * Signature: Subrata, 03/06/22
     */
    public function index(UserInterface $i_user)
    {
        try {
            if (!Auth::check()) return Base::error('Not Authenticated');

            $res = $i_user->getUserProfile();

            if (!$res->success) return Base::error($res->message);

            return Base::success('User profile', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    /**
     * Function update
     * Type: POST
     * Return: Update auth user profile info.
     * Note: 
     * Signature: Subrata, 03/06/22
     */
    public function update(UserInterface $i_user, Request $request)
    {
        try {
            // $validator = Validator::make($request->all(), [
            //     'image_url' => 'required|string',
            //     'name' => 'required|string',
            //     'email' => 'required|string|email',
            //     'phone' => 'required|string',
            //     'address' => 'required|string',
            // ]);
            // if ($validator->fails()) return Base::validation($validator);

            $data = $request->all();
            $user = Auth::user();

            $is_email_duplicate = User::where([['email', $request->email], ['id', '!=', $user->id]])->first();
            if ($is_email_duplicate) return Base::error('Email already taken'); // DONE

            $res = $i_user->updateUser($user, $data);
            if (!$res->success) return Base::error($res->message);

            return Base::success('User updated', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }


    public function changePassword(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'old_password' => 'required',
                'password' => 'required|confirmed|min:6',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = Auth::user();

            // if ($user->login_type != "normal") return Base::error("Reset password option is not available for this user"); // DONE

            $pass_match = Hash::check($request->old_password, $user->password);
            if (!$pass_match) return Base::error("You have entered a wrong password"); // DONE
            if (Hash::check($request->password, $user->password)) return Base::error("Old password cannot be new password"); // DONE

            $res = $i_user->updatePassword($user, $request->password);
            if (!$res->success) return Base::error($res->message);

            return Base::success('Password changed successfully', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function changeLanguage(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'code' => 'required|in:en,ja,bn,zh',
            ]);
            if ($validator->fails()) return Base::validation($validator);

            $user = Auth::user();

            $res = $i_user->updateLanguage($user, $request->code);
            if (!$res->success) return Base::error($res->message);

            return Base::success('Language changed successfully', $res->data); // DONE
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function togglePushOrEmailNotification(TogglePushOrEmailNotificationRequest $request)
    {

        return $this->user_interface->togglePushOrEmailNotification($request);
    }



    public function delete(UserInterface $i_user, Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'reason' => 'required',
            ]);
            if ($validator->fails()) return Base::validation($validator);
            $user = Auth::user();

            $res = $i_user->delete($user, $request);
            if (!$res->success) return Base::error($res->message);

            // $token = Auth::user()->token();
            // $token->revoke();
            $user->tokens->each(function ($token, $key) {
                $token->delete();
            });

            // $refreshTokenRepository = app(\Laravel\Passport\RefreshTokenRepository::class);
            // foreach (User::find($user->id)->tokens as $token) {
            //     $token->revoke();
            //     $refreshTokenRepository->revokeRefreshTokensByAccessTokenId($token->id);
            // }

            return Base::success('User Deleted', $res->data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function getReferCode()
    {
        try {
            $is_referral = Setting::where('type', 'is_referral')->first()->value;
            if($is_referral == 0) return Base::error('Referral is not available');
            $user = Auth::user();

            $refer_code = $user->refer_code;

            return Base::success('Your refer code', $refer_code);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function deleteText(UserInterface $i_user)
    {
        try {
            $data = $i_user->deleteText();
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
    public function vIDVerifyInfo(UserInterface $i_user, Request $request)
    {
        try {
            $data = $i_user->vIDVerifyInfo($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function getVerificationRequests(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'status' => 'required|in:pending,history',
                'type' => 'in:mncard,rcard,driving,passport',
            ]);

            if ($validator->fails()) return Base::validation($validator);
            $take = $request->take ? $request->take : 20;

            $data = UserInfo::join('users', 'users.id', '=', 'user_infos.user_id')
                ->select('user_infos.*', 'users.name', 'users.email', 'users.image_url', 'users.image', 'users.phone')
                ->When(($request->search != null && $request->type == null && $request->status == 'pending'), function ($q) use ($request) {
                    $q->where([['users.name', 'like', '%' . $request->search . '%'], ['status', '=', 'pending']])
                        ->orWhere([['users.email', 'like', '%' . $request->search . '%'], ['status', '=', 'pending']]);
                })
                ->When(($request->search != null && $request->type == null && $request->status == 'history'), function ($q) use ($request) {
                    $q->where([['users.name', 'like', '%' . $request->search . '%'], ['status', '<>', 'pending']])
                        ->orWhere([['users.email', 'like', '%' . $request->search . '%'], ['status', '<>', 'pending']]);
                })
                ->When(($request->search != null && $request->status == 'pending'), function ($q) use ($request) {
                    $q->where([['users.name', 'like', '%' . $request->search . '%'], ['status', '=', 'pending'], ['type', '=', $request->type]])
                        ->orWhere([['users.email', 'like', '%' . $request->search . '%'], ['status', '=', 'pending'], ['type', '=', $request->type]]);
                })
                ->When(($request->search != null && $request->status == 'history'), function ($q) use ($request) {
                    $q->where([['users.name', 'like', '%' . $request->search . '%'], ['status', '<>', 'pending'], ['type', '=', $request->type]])
                        ->orWhere([['users.email', 'like', '%' . $request->search . '%'], ['status', '<>', 'pending'], ['type', '=', $request->type]]);
                })
                ->when($request->type != null, function ($q) use ($request) {
                    $q->where('type', '=', $request->type);
                })
                ->When($request->status == 'pending', function ($q) use ($request) {
                    $q->where('status', '=', 'pending');
                })
                ->When($request->status == 'history', function ($q) use ($request) {
                    $q->where('status', '=', 'verified')
                        ->orWhere('status', '=', 'not_verified');
                })
                ->paginate($take);



            return Base::success('User verification information!', $data);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }

    public function pointDetails(Request $request, UserInterface $i_user)
    {
        try {
            $data = $i_user->pointDetails($request);
            return $data->success ? Base::success($data->message, $data->data) : Base::error($data->message);
        } catch (Exception $e) {
            return Base::exception($e);
        }
    }
}
